Package: FGNet
Type: Package
Title: Functional Gene Networks derived from biological enrichment
        analyses
Description: Build and visualize functional gene and term networks from clustering of enrichment analyses in multiple annotation spaces. The package includes a graphical user interface (GUI) and functions to perform the functional enrichment analysis through DAVID, GeneTerm Linker, gage (GSEA) and topGO.
Version: 3.44.0
Date: 2022-04-08
Author: Sara Aibar, Celia Fontanillo, Conrad Droste and Javier De Las Rivas.
Maintainer: Sara Aibar <saibar@usal.es>
Depends: R (>= 4.2.0)
VignetteBuilder: knitr
Imports: igraph (>= 0.6), hwriter, R.utils, XML, plotrix, reshape2,
        RColorBrewer, png, methods, stats, utils, graphics, grDevices
Suggests: RCurl, gage, topGO, GO.db, reactome.db, RUnit, BiocGenerics,
        org.Sc.sgd.db, knitr, rmarkdown, AnnotationDbi, BiocManager
License: GPL (>= 2)
URL: http://www.cicancer.org
LazyLoad: yes
biocViews: Annotation, GO, Pathways, GeneSetEnrichment, Network,
        Visualization, FunctionalGenomics, NetworkEnrichment,
        Clustering
git_url: https://git.bioconductor.org/packages/FGNet
git_branch: RELEASE_3_22
git_last_commit: 30b9e81
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:57:23 UTC; biocbuild
